import 'package:pip_clients_sessions/pip_clients_sessions.dart';
import 'package:pip_services_sessions/src/data/version1/SessionV1.dart';
import 'package:pip_services3_commons/src/data/PagingParams.dart';
import 'package:pip_services3_commons/src/data/FilterParams.dart';
import 'package:pip_services3_commons/src/data/DataPage.dart';

class SessionsMemoryClientV1 implements ISessionsClientV1 {
  final _maxPageSize = 100;
  var _sessions = <SessionV1>[];

  bool Function(SessionV1) _composeFilter(FilterParams? filter) {
    filter = filter ?? FilterParams();

    var id = filter.getAsNullableString('id');
    var user_id = filter.getAsNullableString('user_id');
    var user_name = filter.getAsNullableString('user_name');
    var active = filter.getAsNullableBoolean('active');
    var client = filter.getAsNullableString('client');

    var ids = filter.getAsObject('ids');
    if (ids is String) {
      ids = ids.split(',');
    }

    if (!(ids is List)) {
      ids = null;
    }

    return (item) {
      if (id != null && item.id != id) {
        return false;
      }

      if (user_id != null && item.user_id != user_id) {
        return false;
      }
      if (user_name != null && item.user_name != user_name) {
        return false;
      }
      if (active != null && item.active != active) {
        return false;
      }
      if (client != null && item.client != client) {
        return false;
      }
      if (ids != null && ids.indexOf(item.id) < 0) {
        return false;
      }
      return true;
    };
  }

  @override
  Future<DataPage<SessionV1>> getSessions(
      String? correlationId, FilterParams filter, PagingParams paging) async {
    var filterDefinition = _composeFilter(filter);
    var sessions = _sessions.where(filterDefinition).toList();

    var skip = paging.getSkip(-1);
    var take = paging.getTake(_maxPageSize);

    var total = 0;
    if (paging.total) {
      total = sessions.length;
    }

    if (skip > 0) {
      sessions = sessions.length > skip ? sessions.sublist(skip) : sessions;
    }

    sessions = sessions.length > take ? sessions.sublist(0, take) : sessions;

    return DataPage(sessions, total);
  }

  @override
  Future<SessionV1?> getSessionById(
      String? correlationId, String sessionId) async {
    SessionV1? session;
    _sessions.forEach((d) {
      session = d.id == sessionId ? d : null;
    });

    return session;
  }

  @override
  Future<SessionV1?> openSession(String? correlationId, String? user_id,
      String? user_name, String? address, String? client, user, data) async {
    var session = SessionV1(
        id: null,
        user_id: user_id,
        user_name: user_name,
        address: address,
        client: client);

    session.user = user;
    session.data = data;

    _sessions.add(session);

    return session;
  }

  @override
  Future<SessionV1?> storeSessionData(
      String? correlationId, String sessionId, data) async {
    SessionV1? session;
    _sessions.forEach((d) {
      session = d.id == sessionId ? d : null;
    });

    if (session != null) {
      session!.data = data;
    }

    return session;
  }

  @override
  Future<SessionV1?> updateSessionUser(
      String? correlationId, String sessionId, user) async {
    SessionV1? session;
    _sessions.forEach((d) {
      session = d.id == sessionId ? d : null;
    });

    if (session != null) {
      session!.user = user;
    }

    return session;
  }

  @override
  Future<SessionV1?> closeSession(
      String? correlationId, String sessionId) async {
    SessionV1? session;
    _sessions.forEach((d) {
      session = d.id == sessionId ? d : null;
    });

    if (session != null) {
      session!.active = false;
    }

    return session;
  }

  @override
  Future<SessionV1?> deleteSessionById(
      String? correlationId, String sessionId) async {
    SessionV1? session;
    _sessions = _sessions.where((d) => d.id != sessionId).toList();

    return session;
  }
}
